# frozen_string_literal: true

require 'singleton'

module  Erratum
  class Configuration
    include Singleton

    DEFAULT_ERROR_MAPPINGS = {
      'ActionController::ParameterMissing'      => '::Erratum::Errors::ParameterMissing',
      'ActionController::UnpermittedParameters' => '::Erratum::Errors::UnpermittedParameters',
      'ActiveRecord::InvalidForeignKey'         => '::Erratum::Errors::InvalidAssociation',
      'ActiveRecord::RecordInvalid'             => '::Erratum::Errors::ResourcePersistence',
      'ActiveRecord::RecordNotFound'            => '::Erratum::Errors::ResourceNotFound',
      'ActiveRecord::RecordNotSaved'            => '::Erratum::Errors::ResourcePersistence',
      'ActiveRecord::RecordNotUnique'           => '::Erratum::Errors::ResourceNotUnique',
      'ActiveRecord::ValueTooLong'              => '::Erratum::Errors::AttributeLength',
    }.freeze

    attr_writer :developer_documentation_urls,
                :external_documentation_urls

    def error_mappings
      @error_mappings ||= DEFAULT_ERROR_MAPPINGS.dup
    end

    def error_mappings=(other)
      @error_mappings = DEFAULT_ERROR_MAPPINGS.merge(other)
    end

    def developer_documentation_urls
      @developer_documentation_urls ||= {}
    end

    def external_documentation_urls
      @external_documentation_urls ||= {}
    end

    def to_h
      {
        'external_documentation_urls'  => external_documentation_urls,
        'developer_documentation_urls' => developer_documentation_urls,
      }
    end
  end

  def self.configure
    yield configuration
  end

  def self.configuration
    Configuration.instance
  end
end
