# frozen_string_literal: true

require 'erratum/error'
require 'erratum/errors/authentication'

module  Erratum
module  Errors
class   DuplicateAuthentication < RuntimeError
  include Error
  include Errors::Authentication

  attr_accessor :provider,
                :provider_user_id,
                :user_id

  def http_status
    409
  end

  def title
    'Duplicate Authentication'
  end

  def detail
    <<~HEREDOC.chomp.tr("\n", ' ')
      The authentication you attempted to register has already been registered
      by another user. We do not currently support allowing multiple users to be
      connected to the same authentication.
    HEREDOC
  end

  def source
    {
      'provider'         => provider,
      'provider_user_id' => provider_user_id,
      'user_id'          => user_id,
    }
  end
end
end
end
