# frozen_string_literal: true

require 'erratum/error'
require 'erratum/errors/authorization'

module  Erratum
module  Errors
class   Forbidden < RuntimeError
  include Error
  include Errors::Authorization

  NON_SPECIFIC_RESOURCE_ACTIONS = %w{index create}.freeze

  attr_accessor :resource_name,
                :resource_id
  attr_writer   :action

  def http_status
    403
  end

  def title
    'Forbidden'
  end

  def detail
    detail_quantity = if NON_SPECIFIC_RESOURCE_ACTIONS.include? action.to_s
                        "#{action} a #{resource_name}"
                      else
                        "#{action} the #{resource_name} with ID #{resource_id}"
                      end

    <<~HEREDOC.chomp.tr("\n", ' ')
      You do not have access to #{detail_quantity}. Providing a different set of
      credentials may potentially allow you access to this resource.
    HEREDOC
  end

  def source
    @source ||= {
      'resource_name' => resource_name,
      'resource_id'   => resource_id,
      'action'        => @action,
    }
  end

  def action
    @action || 'access'
  end
end
end
end
