# frozen_string_literal: true

require 'apple_core/refinements/string'
require 'erratum/error'
require 'erratum/errors/crud'

module  Erratum
module  Errors
class   AttributeLength < RuntimeError
  using   ::AppleCore::Refinements::String

  include Error
  include Errors::Crud

  attr_accessor :pointer,
                :attribute,
                :length

  def self.convert(original_error, overrides = {})
    case original_error.class.name
    when 'ActiveRecord::ValueTooLong'
      message_info_pattern = /ERROR:  value too long for type character varying\((\d+)\)/
      message_info         = original_error
                               .message
                               .match(message_info_pattern)
                               .captures

      initialization_parameters = {
        attribute: nil,
        pointer:   nil,
        length:    message_info[0],
      }

      new(initialization_parameters.merge(overrides))
    end
  end

  def http_status
    422
  end

  def title
    'Value Too Long'
  end

  def detail
    @detail || <<~HEREDOC.chomp.tr("\n", ' ')
      One or more of the attributes you've attempted to set is longer than the
      maximum length of #{length} characters allowed.
    HEREDOC
  end

  def source
    super.merge(
      'pointer'   => pointer,
      'parameter' => attribute,
      'length'    => length,
    )
  end
end
end
end
