# frozen_string_literal: true

require 'erratum/error'
require 'erratum/errors/crud'

module  Erratum
module  Errors
class   InvalidAssociation < RuntimeError
  include Error
  include Errors::Crud

  attr_accessor :association_name,
                :association_id

  def self.convert(original_error, overrides = {})
    initialization_parameters = {}

    case original_error.class.name
    when 'ActiveRecord::InvalidForeignKey'
      message_info_pattern = /DETAIL:  Key \((.*?)(?:_id)?\)=\(([a-f0-9\-]+)\)/
      message_info         = original_error
                               .message
                               .match(message_info_pattern)
                               .captures

      initialization_parameters = {
        association_name: message_info[0],
        association_id:   message_info[1],
      }
    end

    new(initialization_parameters.merge(overrides))
  end

  def http_status
    422
  end

  def title
    'Association Error'
  end

  def detail
    if association_name == 'id'
      <<~HEREDOC.chomp.tr("\n", ' ')
        You cannot #{action} this #{resource_name} because its
        #{association_name} is still being referenced by other things.
      HEREDOC
    else
      <<~HEREDOC.chomp.tr("\n", ' ')
        The #{association_name} that you attempted to associate with the
        #{resource_name} was not valid.
      HEREDOC
    end
  end

  def source
    super.merge(
      'association'    => association_name,
      'association_id' => association_id,
    )
  end
end
end
end
