# frozen_string_literal: true

require 'apple_core/refinements/string'
require 'erratum/error'
require 'erratum/errors/crud'

module  Erratum
module  Errors
class   ResourceNotUnique < RuntimeError
  using   ::AppleCore::Refinements::String

  include Error
  include Errors::Crud

  attr_accessor :pointer,
                :attribute,
                :value

  def self.convert(original_error, overrides = {})
    case original_error.class.name
    when 'ActiveRecord::RecordNotUnique'
      message_info_pattern = /DETAIL:  Key \((.+)\)=\((.+)\) already exists\./
      message_info         = original_error
                               .message
                               .match(message_info_pattern)
                               .captures

      initialization_parameters = {
        attribute: message_info[0],
        pointer:   "/data/attributes/#{message_info[0]}",
        value:     message_info[1],
      }

      new(initialization_parameters.merge(overrides))
    end
  end

  def http_status
    422
  end

  def title
    'Resource Not Unique'
  end

  def detail
    @detail || <<~HEREDOC.chomp.tr("\n", ' ')
      #{attribute.to_s.humanize} has already been used.
    HEREDOC
  end

  def source
    super.merge(
      'pointer'   => pointer,
      'parameter' => attribute,
      'value'     => value,
    )
  end
end
end
end
