# frozen_string_literal: true

require 'erratum/error'
require 'erratum/errors/crud'

module  Erratum
module  Errors
class   ParameterMissing < RuntimeError
  include Error
  include Errors::Crud

  attr_accessor :parameter

  def self.convert(original_error, overrides = {})
    initialization_parameters = {}

    case original_error.class.name
    when 'ActionController::ParameterMissing'
      initialization_parameters = {
        parameter: original_error.param,
      }
    end

    new(initialization_parameters.merge(overrides))
  end

  def http_status
    400
  end

  def title
    'Missing Parameter'
  end

  def detail
    <<~HEREDOC.chomp.tr("\n", ' ')
      When attempting to #{action} a #{resource_name}, '#{parameter}' is
      a required parameter.
    HEREDOC
  end

  def source
    super.merge(
      'required_parameter' => parameter,
    )
  end
end
end
end
