# frozen_string_literal: true

require 'erratum/error'
require 'erratum/errors/crud'

module  Erratum
module  Errors
class   UnpermittedParameters < RuntimeError
  include Error
  include Errors::Crud

  attr_accessor :parameters

  def self.convert(original_error, overrides = {})
    initialization_parameters = {}

    case original_error.class.name
    when 'ActionController::UnpermittedParameters'
      initialization_parameters = {
        parameters: Array(original_error.params),
      }
    end

    new(initialization_parameters.merge(overrides))
  end

  def initialize(**attrs)
    self.parameters = Array(attrs.delete(:parameters))

    super(**attrs)
  end

  def http_status
    400
  end

  def title
    'Unpermitted Parameters'
  end

  def detail
    "Attempting to #{action} a #{resource_name} with the following parameters is " \
    "not allowed: #{parameters.join(', ')}"
  end

  def source
    super.merge(
      'unpermitted_parameters' => parameters,
    )
  end
end
end
end
